<#
.SYNOPSIS
    Creates or Removes Registry Keys for all Java Versions.
.VERSION
    2.0
.DESCRIPTION
    Creates or Deletes a set of registry keys used by all of the known versions of Java, as well as potential future ones. 
    This is used in sequencing of Java JRE packages in Microsoft App-V.

    You run this script once, prior to sequencing, to add the entries.
    Then, when you begin monitoring mode, you run it again to remove them, followed by installation of the Java version that you need.

    On an x64 system this should be run in a 32-bit powershell window so that they keys match that of the 32-bit java!

    See http:/www.tmurgent.com/appv/index.php/resources/tools/322-jreblock for more information.
    
.PARAMETER addBool
    True means to add the registry entries
    False means to remove them.
    Mandatory bool
.PARAMETER majorVersionMax
    First two levels of the largest version to perform on.  For example "18" for 1.8.y.zz
    Optional int
    Default = 19
.PARAMETER middleVersionMax
    Middle level of the largest version to perform on.  For example 1 for w.x.1.zz
    Optional Int
    Default = 2
.PARAMETER minorVersionMax
    Update level of the largest version to perform on.  For example 99 for w.x.y.99
    Optional Int
    Default = 99

.EXAMPLE
    PowerShell.exe -ExecutionPolicy ByPass -File jreBlock.ps1 $true 

    Creates all keys from 1.3.0.0 up to  version 1.8.0.199 and then 1.9.2.99
.EXAMPLE
    PowerShell.exe  -ExecutionPolicy ByPass -File jreBlock.ps1 $false 18 0 30

    Removes all keys up to version 1.8.0.30
#>



Param (
        [parameter( Position = 0, Mandatory=$True ) ] [ValidateNotNullOrEmpty()] [bool]$addBool,
		[parameter( Position = 1, Mandatory=$False )]        [int]$majorVersionMax,
		[parameter( Position = 2, Mandatory=$False )]        [int]$middleVersionMax,
		[parameter( Position = 3, Mandatory=$False )]        [int]$minorVersionMax
        )
		
		function regAdd([string]$HKwhich, [int]$majorVersion, [int]$minorVersion, [int]$middleVersion, [bool]$addBool=$true) {
		<#	Param (
				[parameter( Position = 0,
							Mandatory=$true )
				]
				[ValidateNotNullOrEmpty()]
				[string]$HKwhich,
				[parameter( Position = 0,
							Mandatory=$true )
				]
				[ValidateNotNullOrEmpty()]
				[int]$majorVersion,
				
				[parameter( Position = 1,
							Mandatory=$true )
				]
				[ValidateNotNullOrEmpty()]
				[int]$minorVersion,
				
				[parameter( Position = 2 )
				]
				[bool]$addBool=$true
				
				)
		#>
			if($addBool) {
				[string]$add='add'
			} else {
				[string]$add='remove'
			}
			[int]$yavnCurrent = $majorVersion - 7
			
			[string]$path= "$($HKwhich):\Software\classes\CLSID"												#should I be doing the keys under HKey_Users as well?
			[string]$keyFront= '{CAFEEFAC-00' + $majorVersion + '-000' + $middleVersion +'-00' + $minorVersion + '-ABCDEFFEDCB'    #may be able to simplify if var names cannot contain '-'
			if($minorVersion -lt 10) {
				$keyFront= '{CAFEEFAC-00' + $majorVersion + '-000' + $middleVersion +'-000' + $minorVersion + '-ABCDEFFEDCB'
			}
			if($minorVersion -gt 99) {
				$keyFront= '{CAFEEFAC-00' + $majorVersion + '-000' + $middleVersion +'-0' + $minorVersion + '-ABCDEFFEDCB'
			}
			[string]$key= $keyFront + 'A}'
			if($addBool) {
				New-Item -path $path -name $key  -Value ('Java Plug-in 1.' + ($majorVersion - 10) + '.'+ $middleVersion + '_' + $minorVersion) -force
			} else {
				Remove-Item -path ($path + '\' + $key) -recurse
			}
			$key= $keyFront + 'B}'
			if($addBool) {
				New-Item -path $path -name $key  -Value ('Java Plug-in 1.' + ($majorVersion - 10) + '.'+ $middleVersion + '_' + $minorVersion) -force
			} else {
				Remove-Item -path ($path + '\' + $key) -recurse
			}
			$key= $keyFront + 'C}'
			if($addBool) {
				New-Item -path $path -name $key  -Value ('Java Plug-in 1.' + ($majorVersion - 10) + '.'+ $middleVersion + '_' + $minorVersion) -force
			} else {
				Remove-Item -path ($path + '\' + $key) -recurse
			}
			if($majorVersion -ge 15) {		#lower versions will generate errors due to multiple middle versions
				if($addBool) {
					
					New-Item -path "$($HKwhich):\SOFTWARE\Classes" -Name ('JavaPlugin.' + $yavnCurrent + $minorVersion + 2) -force		#should I be creating the parent key or CLSID under it?
				} else {
					Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\JavaPlugin.$($yavnCurrent)$($minorVersion)2" -recurse
				}
			}
			
			
		}
		
		function versionIndependent([string]$HKwhich, [bool]$addBool=$true, [int]$majorVersionExclude=8, [int]$minorVersionExclude=45) {
			
			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name '{4299124F-F2C3-41b4-9C73-9236B2AD0E8F}' -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{4299124F-F2C3-41b4-9C73-9236B2AD0E8F}" -recurse
			}
			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name '{5852F5ED-8BF4-11D4-A245-0080C6F74284}' -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{5852F5ED-8BF4-11D4-A245-0080C6F74284}" -recurse
			}
			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name '{761497BB-D6F0-462C-B6EB-D4DAF1D92D43}' -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{761497BB-D6F0-462C-B6EB-D4DAF1D92D43}" -recurse
			}
			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name '{8AD9C840-044E-11D1-B3E9-00805F499D93}' -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{8AD9C840-044E-11D1-B3E9-00805F499D93}" -recurse
			}
			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name '{08B0E5C0-4FCB-11CF-AAA5-00401C608501}' -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{08B0E5C0-4FCB-11CF-AAA5-00401C608501}" -recurse
			}
			[int]$a=13
			[int]$b
			while($a -le 19) {
				if($addBool) {
					New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name "{CAFEEFAC-00$a-0000-FFFF-ABCDEFFEDCBA}" -force
				} else {
					Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{CAFEEFAC-00$a-0000-FFFF-ABCDEFFEDCBA}" -recurse
				}
				$b = $a - 10
				if($addBool) {
					New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name "{CAFEEFAC-DEC$b-0000-0001-ABCDEFFEDCBA}" -force
				} else {
					Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{CAFEEFAC-DEC$b-0000-0001-ABCDEFFEDCBA}" -recurse
				}
				if($addBool) {
					New-Item -Path "$($HKwhich):\SOFTWARE\Classes" -Name "JavaWebStart.isInstalled.1.$b.0.0" -force
				} else {
					Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\JavaWebStart.isInstalled.1.$b.0.0" -recurse
				}
				if($addBool) {
					New-Item -Path "$($HKwhich):\SOFTWARE\Microsoft\Internet Explorer\ActiveX Compatibility" -Name "{CAFEEFAC-DEC$b-0000-0000-ABCDEFFEDCBA}" -force
				} else {
					Remove-Item -path "$($HKwhich):\SOFTWARE\Microsoft\Internet Explorer\ActiveX Compatibility\{CAFEEFAC-DEC$b-0000-0000-ABCDEFFEDCBA}" -recurse
				}
				$a= $a + 1
				
			}
			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name '{CAFEEFAC-FFFF-FFFF-FFFF-ABCDEFFEDCBA}' -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{CAFEEFAC-FFFF-FFFF-FFFF-ABCDEFFEDCBA}" -recurse
			}
			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name '{DBC80044-A445-435b-BC74-9C25C1C588A9}' -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{DBC80044-A445-435b-BC74-9C25C1C588A9}" -recurse
			}
			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE\Classes\CLSID" -Name '{E19F9331-3110-11D4-991C-005004D3B3DB}' -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\CLSID\{E19F9331-3110-11D4-991C-005004D3B3DB}" -recurse
			}


            [string[]]$ClassesKeys = ".jar",".jnlp","jarfile","JavaPlugin","JavaWebStart.isInstalled","JavaPlugin.160_07","JavaPlugin.FamilyVersionSupport","JNLPFile","JavaSoft","JreMetrics"
            foreach ($ck in $ClassesKeys)
            {
			    if($addBool) {
				    New-Item -Path "$($HKwhich):\SOFTWARE\Classes" -Name "$ck" -force
			    } else {
				    Remove-Item -path "$($HKwhich):\SOFTWARE\Classes\$ck" -recurse
			    }			
			}

            [string[]]$AppKeys = "java-deployment-toolkit", "x-java-applet", "x-java-jnlp-file"
            foreach ($ak in $AppKeys)
            { 
			    if($addBool) {
				    New-Item -Path "$($HKwhich):\SOFTWARE\MIME\Database\Content Type\application" -Name "$ak" -force
			    } else {
				    Remove-Item -path "$($HKwhich):\SOFTWARE\MIME\Database\Content Type\application\$ak" -recurse
			    }
			}

			if($addBool) {
				New-Item -Path "$($HKwhich):\SOFTWARE" -Name "Netscape" -force
			} else {
				Remove-Item -path "$($HKwhich):\SOFTWARE\Netscape" -recurse
			}

			
			
			[int]$yavn=$majorVersionExclude-7
            if ($HKwhich -eq "HKCU") {
			    if($addBool -eq $false) {				#some of this is only necessary for java 7+, but should not cause harm    # should probably have a separate function for stuff like this
												#confirm that var names cannot include a '.' 
				    New-Item -Path HKCU:\Software\AppDataLow\Software\JavaSoft -Name DeploymentProperties -force
				    Set-ItemProperty -Path HKCU:\Software\AppDataLow\Software\JavaSoft\DeploymentProperties -Name "deployment.expiration.decision.$yavn.$minorVersionExclude.2" -Value 'later'
				    Set-ItemProperty -Path HKCU:\Software\AppDataLow\Software\JavaSoft\DeploymentProperties -Name "deployment.expiration.decision.suppression.$yavn.$minorVersionExclude.2" -Value 'true'
			    }
			}
		}
		
		function callAdd([string]$HKWhich,[bool]$addBool=$true, [int]$majorVersion, [int]$middleVersion, [int]$minorVersionMin=0, [int]$minorVersionMax=99) {
			[int]$minorVersionCurrent=$minorVersionMin
			while($minorVersionCurrent -le $minorVersionMax) {
				regAdd $HKwhich $majorVersion $minorVersionCurrent $middleVersion $addBool
				$minorVersionCurrent= $minorVersionCurrent + 1
			}			
		}
		
#These are no longer used in the external interface.  The values below render then useless.
$minorVersionExclude=99
$majorVersionExclude=99

# Defaults for things that can be changed
$majorVersionMax = 19
$middleVersionMax = 2
$minorVersionMax=99		
		
		#[bool]$addBool=$true   #replace
		[int]$majorVersionCurrent=13
		[int]$minorVersionCurrent=1
		[int]$middleVersionCurrent=0
		if($majorVersionExclude -lt 10) {		
			$majorVersionExclude = $majorVersionExclude + 10	#versions are usually listed as something like 1.3u15. For use in registry, I break this into major version 13 minor version 15, but it is normal to call it version 3. This converts that into the form I want.
		}
		if($majorVersionMax -lt 10) {		
			$majorVersionMax = $majorVersionMax + 10	#needs to use the same format as $majorVersionCurrent
		}		#should I check for input like 1.3 instead of just 3? not sure how that would interact with type int in powershell anyway
		
		#[int]$yavn=$majorVersionExclude-7
		#[string]$path
		#[string]$key
		
		callAdd "HKLM" $addBool 13 0 0 5
		callAdd "HKLM" $addBool 13 1 0 27
		callAdd "HKLM" $addBool 14 0 0 4
		callAdd "HKLM" $addBool 14 1 0 7
		callAdd "HKLM" $addBool 14 2 0 30
		callAdd "HKLM" $addBool 15 0 0 75
		callAdd "HKLM" $addBool 16 0 0 85
		if($majorVersionMax -eq 17) {
			callAdd "HKLM" $addBool 17 0 0 $minorVersionMax
		}
		if($majorVersionMax -gt 17) {
			callAdd "HKLM" $addBool 17 0 0 99
		}
		if($majorVersionMax -eq 18) {
			callAdd "HKLM" $addBool 18 0 0 $minorVersionMax
		}
		if($majorVersionMax -gt 18) {
			callAdd "HKLM" $addBool 18 0 0 199
		}
		if($majorVersionMax -eq 19) {
			callAdd "HKLM" $addBool 19 0 0 $minorVersionMax
		}
		if($majorVersionMax -gt 19) {
			callAdd "HKLM" $addBool 19 0 0 99
			callAdd "HKLM" $addBool $majorVersionMax 0 0 $minorVersionMax
		}
		
		

		callAdd "HKCU" $addBool 13 0 0 5
		callAdd "HKCU" $addBool 13 1 0 27
		callAdd "HKCU" $addBool 14 0 0 4
		callAdd "HKCU" $addBool 14 1 0 7
		callAdd "HKCU" $addBool 14 2 0 30
		callAdd "HKCU" $addBool 15 0 0 75
		callAdd "HKCU" $addBool 16 0 0 85
		if($majorVersionMax -eq 17) {
			callAdd "HKCU" $addBool 17 0 0 $minorVersionMax
		}
		if($majorVersionMax -gt 17) {
			callAdd "HKCU" $addBool 17 0 0 99
		}
		if($majorVersionMax -eq 18) {
			callAdd "HKCU" $addBool 18 0 0 $minorVersionMax
		}
		if($majorVersionMax -gt 18) {
			callAdd "HKCU" $addBool 18 0 0 99
		}
		if($majorVersionMax -eq 19) {
			callAdd "HKCU" $addBool 19 0 0 $minorVersionMax
		}
		if($majorVersionMax -gt 19) {
			callAdd "HKCU" $addBool 19 0 0 99
			callAdd "HKCU" $addBool $majorVersionMax 0 0 $minorVersionMax
		}
		
		
		

		versionIndependent "HKLM" $addBool $majorVersionExclude $minorVersionExclude
		versionIndependent "HKCU" $addBool $majorVersionExclude $minorVersionExclude
		